//  gmodconfig - a Gnome based kernel module configuration tool
//  http://gmodconfig.sourceforge.net/
//  Copyright (C) 2003 Cyril Bortolato
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//  dkmsinstallmgr.cpp: DKMS installation manager
//--------------------------------------------------------------------------//

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "dkmsinstallmgr.h"
#include <iostream>
#include "pipereader.h"
#include <sys/stat.h>
#include <errno.h>

using namespace gmodcfg;
using namespace std;

//--------------------------------------------------------------------------//
//  C'tor
//--------------------------------------------------------------------------//
DkmsInstallManager::DkmsInstallManager( GtkWidget* appWidget, int verbosity )
    : InstallManager( appWidget, verbosity ) {}

//--------------------------------------------------------------------------//
//  Install a tar.gz DKMS package
//--------------------------------------------------------------------------//
bool
DkmsInstallManager::installPackage( const string& pkgFile )
{
    UpdateProgressBar( 0 );

    if ( Verbosity )
        cout << "DKMS InstallManager: install package " << pkgFile << endl;

    // Check existence of the dkms.conf and install.sh files in the package
    string cmd1 = "tar ztf " + pkgFile;
    if ( Verbosity >= 2 )
        cout << cmd1 << endl;

    ifstream ifs;
    PipeReader pr( cmd1, ifs );

    string confFile = "dkms.conf";
    string installScriptName = "/install.sh", installScript;
    bool hasConfFile = false;
    string line;

    while ( getline( ifs, line ) )
    {
        if ( Verbosity >= 3 )
            cout << line << endl;

        if ( line.find( confFile ) == line.length() - confFile.length() )
            hasConfFile = true;

        else if ( line.find( installScriptName ) ==
                                line.length() - installScriptName.length() )
            installScript = line;
    }

    if ( !hasConfFile )
    {
        WriteStatusBarMessage( "This is not a DKMS package (can't find dkms.conf)" );
        return false;
    }

    if ( installScript.empty() )
    {
        WriteStatusBarMessage( "Sorry, I cannot install this package (need install.sh script)" );
        return false;
    }

    UpdateProgressBar( 0.33 );
 
    // Uncompress and un-tar this file
    string cmd2 = "tar zxf " + pkgFile + " -C /tmp";
    if ( Verbosity >= 2 )
        cout << cmd2 << endl;

    if ( system( cmd2.c_str() ) )
    {
        WriteStatusBarMessage( "Failed to uncompress/untar the package" );
        return false;
    }

    UpdateProgressBar( 0.66 );

    // Execute the install.sh script
    string cmd3 = "/tmp/" + installScript;
    if ( Verbosity >= 2 )
        cout << cmd3 << endl;

    if ( system( cmd3.c_str() ) )
    {
        WriteStatusBarMessage( "Execution of install.sh script failed" );
        return false;
    }
 
    UpdateProgressBar( 1.0 );

    return true;
}

//--------------------------------------------------------------------------//
//  Uninstall a package
//--------------------------------------------------------------------------//
bool
DkmsInstallManager::uninstallPackage( const string& pkgName )
{
    if ( Verbosity )
        cout << "DkmsInstallManager: uninstallPackage " << pkgName << endl;
    return false;
}

//--------------------------------------------------------------------------//
//  Check if a package is installed
//--------------------------------------------------------------------------//
bool
DkmsInstallManager::isPackageInstalled( const string& pkgName ) const
{
    if ( Verbosity )
        cout << "DkmsInstallManager: isPackageInstalled " << pkgName << endl;
    return false;
}

