//  gmodconfig - a Gnome based kernel module configuration tool
//  http://gmodconfig.sourceforge.net/
//  Copyright (C) 2003 Cyril Bortolato
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//  testreadallmods.cpp: unit test program for the modinfo parser
//--------------------------------------------------------------------------//

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <libgnomevfs/gnome-vfs.h>
#include <iostream>
#include "modinforeader.h"
#include "moduletypes.h"
#include <unistd.h>
#include <sys/utsname.h>
#include <sys/types.h>
#include <sys/dir.h>
#include <sys/param.h>

using namespace gmodcfg;
using namespace std;

static int
filterFiles( const struct direct* entry )
{
    return ! ( (strcmp(entry->d_name, ".") == 0)
            || (strcmp(entry->d_name, "..") == 0) );
}

static void
searchModules( const string& pathName )
{
    struct direct** namelist;
    int count = scandir( pathName.c_str(), &namelist, filterFiles, 0 );
    const char* itemName = basename( pathName.c_str() );
 
    if ( count <= 0 )
    {
        string str( itemName );
        int dot = str.find_last_of(".");
        if ( dot != string::npos && str.substr( dot, 2 ) == ".o" )
            str.erase( dot, 2 );

        //cout << str << endl;
        ModuleInfoReader reader;
        Module* mod = reader.ReadFromModinfo( str );
        delete mod;
    }
    else
        for( int i=0; i<count; ++i )
        {
            const char* childName = namelist[i]->d_name;
            string childPath = pathName + "/" + childName;
            searchModules( childPath );
        }
}

int main()
{
    gnome_vfs_init();

    cout << "Be patient..." << endl;

    // Determine the top modules dir
    struct utsname unameData;
    uname( &unameData );
    string modDir = "/lib/modules/";
    modDir += unameData.release;
    modDir += "/kernel";

    // Run the ModuleInfoReader's modinfo parser on all modules
    searchModules( modDir );

    return 0;
}

